@@ -201,12 +201,12 @@ private:
 
     TwoByteOpcodeID jccRel32(Condition cond)
     {
-        return (TwoByteOpcodeID)(OP2_JCC_rel32 + cond);
+        return (TwoByteOpcodeID)(int(OP2_JCC_rel32) + cond);
     }
 
     TwoByteOpcodeID setccOpcode(Condition cond)
     {
-        return (TwoByteOpcodeID)(OP_SETCC + cond);
+        return (TwoByteOpcodeID)(int(OP_SETCC) + cond);
     }
 
     typedef enum {
@@ -255,47 +255,6 @@ public:
     {
     }
 
-#if defined(V4_BOOTSTRAP)
-    template <typename LabelType>
-    class Jump {
-        template<class TemplateAssemblerType>
-        friend class AbstractMacroAssembler;
-        friend class Call;
-        template <typename, template <typename> class> friend class LinkBufferBase;
-    public:
-        Jump()
-        {
-        }
-
-        Jump(AssemblerLabel jmp)
-            : m_label(jmp)
-        {
-        }
-
-        LabelType label() const
-        {
-            LabelType result;
-            result.m_label = m_label;
-            return result;
-        }
-
-        void link(AbstractMacroAssembler<X86Assembler>* masm) const
-        {
-            masm->m_assembler.linkJump(m_label, masm->m_assembler.label());
-        }
-
-        void linkTo(LabelType label, AbstractMacroAssembler<X86Assembler>* masm) const
-        {
-            masm->m_assembler.linkJump(m_label, label.label());
-        }
-
-        bool isSet() const { return m_label.isSet(); }
-
-    private:
-        AssemblerLabel m_label;
-    };
-#endif
-
     // Stack operations:
 
     void push_r(RegisterID reg)
