@@ -95,8 +95,9 @@ private:
         QSizeF array[Qt::NSizeHints];
     } SizeHints;
 
-    mutable QVector<SizeHints> m_cachedItemSizeHints;
+    mutable QHash<QQuickItem*, SizeHints> m_cachedItemSizeHints;
     mutable QSizeF m_cachedSizeHints[Qt::NSizeHints];
+    SizeHints &cachedItemSizeHints(int index) const;
 };
 
 class QQuickStackLayoutPrivate : public QQuickLayoutPrivate
