@@ -42,6 +42,8 @@
 #include <private/qv4lookup_p.h>
 #include <private/qv4generatorobject_p.h>
 
+#if QT_CONFIG(qml_jit)
+
 QT_USE_NAMESPACE
 using namespace QV4;
 using namespace QV4::JIT;
@@ -538,6 +540,8 @@ void BaselineJIT::generate_ThrowException()
     as->passEngineAsArg(0);
     BASELINEJIT_GENERATE_RUNTIME_CALL(ThrowException, CallResultDestination::Ignore);
     as->gotoCatchException();
+
+    // LOAD_ACC(); <- not needed here since it would be unreachable.
 }
 
 void BaselineJIT::generate_GetException() { as->getException(); }
@@ -545,9 +549,11 @@ void BaselineJIT::generate_SetException() { as->setException(); }
 
 void BaselineJIT::generate_CreateCallContext()
 {
+    STORE_ACC();
     as->prepareCallWithArgCount(1);
     as->passCppFrameAsArg(0);
     BASELINEJIT_GENERATE_RUNTIME_CALL(PushCallContext, CallResultDestination::Ignore);
+    LOAD_ACC();
 }
 
 void BaselineJIT::generate_PushCatchContext(int index, int name) { as->pushCatchContext(index, name); }
@@ -930,3 +936,6 @@ void BaselineJIT::endInstruction(Instr::Type instr)
 {
     Q_UNUSED(instr);
 }
+
+#endif // QT_CONFIG(qml_jit)
+
