@@ -41,7 +41,7 @@
 
 #include "qqmljsastvisitor_p.h"
 
-QT_QML_BEGIN_NAMESPACE
+QT_BEGIN_NAMESPACE
 
 namespace QQmlJS { namespace AST {
 
@@ -105,6 +105,12 @@ ClassExpression *Node::asClassDefinition()
     return nullptr;
 }
 
+bool Node::ignoreRecursionDepth() const
+{
+    static const bool doIgnore = qEnvironmentVariableIsSet("QV4_CRASH_ON_STACKOVERFLOW");
+    return doIgnore;
+}
+
 ExpressionNode *ExpressionNode::expressionCast()
 {
     return this;
@@ -131,7 +137,7 @@ FormalParameterList *ExpressionNode::reparseAsFormalParameterList(MemoryPool *po
     }
     AST::PatternElement *binding = nullptr;
     if (AST::IdentifierExpression *idExpr = AST::cast<AST::IdentifierExpression *>(expr)) {
-        binding = new (pool) AST::PatternElement(idExpr->name, rhs);
+        binding = new (pool) AST::PatternElement(idExpr->name, /*type annotation*/nullptr, rhs);
         binding->identifierToken = idExpr->identifierToken;
     } else if (AST::Pattern *p = expr->patternCast()) {
         SourceLocation loc;
@@ -161,7 +167,7 @@ UiObjectMember *UiObjectMember::uiObjectMemberCast()
     return this;
 }
 
-void NestedExpression::accept0(Visitor *visitor)
+void NestedExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -179,7 +185,7 @@ ClassExpression *NestedExpression::asClassDefinition()
     return expression->asClassDefinition();
 }
 
-void ThisExpression::accept0(Visitor *visitor)
+void ThisExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -187,7 +193,7 @@ void ThisExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void IdentifierExpression::accept0(Visitor *visitor)
+void IdentifierExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -195,7 +201,7 @@ void IdentifierExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NullExpression::accept0(Visitor *visitor)
+void NullExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -203,7 +209,7 @@ void NullExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TrueLiteral::accept0(Visitor *visitor)
+void TrueLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -211,7 +217,7 @@ void TrueLiteral::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void FalseLiteral::accept0(Visitor *visitor)
+void FalseLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -219,7 +225,7 @@ void FalseLiteral::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void SuperLiteral::accept0(Visitor *visitor)
+void SuperLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -228,7 +234,7 @@ void SuperLiteral::accept0(Visitor *visitor)
 }
 
 
-void StringLiteral::accept0(Visitor *visitor)
+void StringLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -236,7 +242,7 @@ void StringLiteral::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TemplateLiteral::accept0(Visitor *visitor)
+void TemplateLiteral::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (TemplateLiteral *it = this; it && accepted; it = it->next) {
@@ -245,7 +251,7 @@ void TemplateLiteral::accept0(Visitor *visitor)
     }
 }
 
-void NumericLiteral::accept0(Visitor *visitor)
+void NumericLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -253,7 +259,7 @@ void NumericLiteral::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void RegExpLiteral::accept0(Visitor *visitor)
+void RegExpLiteral::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -261,7 +267,7 @@ void RegExpLiteral::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ArrayPattern::accept0(Visitor *visitor)
+void ArrayPattern::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this))
         accept(elements, visitor);
@@ -281,7 +287,7 @@ bool ArrayPattern::isValidArrayLiteral(SourceLocation *errorLocation) const {
     return true;
 }
 
-void ObjectPattern::accept0(Visitor *visitor)
+void ObjectPattern::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(properties, visitor);
@@ -456,7 +462,7 @@ bool PatternProperty::convertLiteralToAssignmentPattern(MemoryPool *pool, Source
 }
 
 
-void Elision::accept0(Visitor *visitor)
+void Elision::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         // ###
@@ -465,7 +471,7 @@ void Elision::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void IdentifierPropertyName::accept0(Visitor *visitor)
+void IdentifierPropertyName::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -473,7 +479,7 @@ void IdentifierPropertyName::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void StringLiteralPropertyName::accept0(Visitor *visitor)
+void StringLiteralPropertyName::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -481,7 +487,7 @@ void StringLiteralPropertyName::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NumericLiteralPropertyName::accept0(Visitor *visitor)
+void NumericLiteralPropertyName::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -511,7 +517,7 @@ QString NumericLiteralPropertyName::asString()const
     return locale.toString(id, 'g', 16);
 }
 
-void ArrayMemberExpression::accept0(Visitor *visitor)
+void ArrayMemberExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -521,7 +527,7 @@ void ArrayMemberExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void FieldMemberExpression::accept0(Visitor *visitor)
+void FieldMemberExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -530,7 +536,7 @@ void FieldMemberExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NewMemberExpression::accept0(Visitor *visitor)
+void NewMemberExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -540,7 +546,7 @@ void NewMemberExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NewExpression::accept0(Visitor *visitor)
+void NewExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -549,7 +555,7 @@ void NewExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void CallExpression::accept0(Visitor *visitor)
+void CallExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -559,7 +565,7 @@ void CallExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ArgumentList::accept0(Visitor *visitor)
+void ArgumentList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (ArgumentList *it = this; it; it = it->next) {
@@ -570,7 +576,7 @@ void ArgumentList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void PostIncrementExpression::accept0(Visitor *visitor)
+void PostIncrementExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -579,7 +585,7 @@ void PostIncrementExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void PostDecrementExpression::accept0(Visitor *visitor)
+void PostDecrementExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -588,7 +594,7 @@ void PostDecrementExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void DeleteExpression::accept0(Visitor *visitor)
+void DeleteExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -597,7 +603,7 @@ void DeleteExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void VoidExpression::accept0(Visitor *visitor)
+void VoidExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -606,7 +612,7 @@ void VoidExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TypeOfExpression::accept0(Visitor *visitor)
+void TypeOfExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -615,7 +621,7 @@ void TypeOfExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void PreIncrementExpression::accept0(Visitor *visitor)
+void PreIncrementExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -624,7 +630,7 @@ void PreIncrementExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void PreDecrementExpression::accept0(Visitor *visitor)
+void PreDecrementExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -633,7 +639,7 @@ void PreDecrementExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UnaryPlusExpression::accept0(Visitor *visitor)
+void UnaryPlusExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -642,7 +648,7 @@ void UnaryPlusExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UnaryMinusExpression::accept0(Visitor *visitor)
+void UnaryMinusExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -651,7 +657,7 @@ void UnaryMinusExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TildeExpression::accept0(Visitor *visitor)
+void TildeExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -660,7 +666,7 @@ void TildeExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NotExpression::accept0(Visitor *visitor)
+void NotExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -669,7 +675,7 @@ void NotExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void BinaryExpression::accept0(Visitor *visitor)
+void BinaryExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(left, visitor);
@@ -679,7 +685,7 @@ void BinaryExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ConditionalExpression::accept0(Visitor *visitor)
+void ConditionalExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -690,7 +696,7 @@ void ConditionalExpression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void Expression::accept0(Visitor *visitor)
+void Expression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(left, visitor);
@@ -700,7 +706,7 @@ void Expression::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void Block::accept0(Visitor *visitor)
+void Block::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statements, visitor);
@@ -709,7 +715,7 @@ void Block::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void StatementList::accept0(Visitor *visitor)
+void StatementList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (StatementList *it = this; it; it = it->next) {
@@ -720,7 +726,7 @@ void StatementList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void VariableStatement::accept0(Visitor *visitor)
+void VariableStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(declarations, visitor);
@@ -729,7 +735,7 @@ void VariableStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void VariableDeclarationList::accept0(Visitor *visitor)
+void VariableDeclarationList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (VariableDeclarationList *it = this; it; it = it->next) {
@@ -740,7 +746,7 @@ void VariableDeclarationList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void EmptyStatement::accept0(Visitor *visitor)
+void EmptyStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -748,7 +754,7 @@ void EmptyStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ExpressionStatement::accept0(Visitor *visitor)
+void ExpressionStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -757,7 +763,7 @@ void ExpressionStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void IfStatement::accept0(Visitor *visitor)
+void IfStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -768,7 +774,7 @@ void IfStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void DoWhileStatement::accept0(Visitor *visitor)
+void DoWhileStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statement, visitor);
@@ -778,7 +784,7 @@ void DoWhileStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void WhileStatement::accept0(Visitor *visitor)
+void WhileStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -788,7 +794,7 @@ void WhileStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ForStatement::accept0(Visitor *visitor)
+void ForStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(initialiser, visitor);
@@ -801,7 +807,7 @@ void ForStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ForEachStatement::accept0(Visitor *visitor)
+void ForEachStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(lhs, visitor);
@@ -812,7 +818,7 @@ void ForEachStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ContinueStatement::accept0(Visitor *visitor)
+void ContinueStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -820,7 +826,7 @@ void ContinueStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void BreakStatement::accept0(Visitor *visitor)
+void BreakStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -828,7 +834,7 @@ void BreakStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ReturnStatement::accept0(Visitor *visitor)
+void ReturnStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -837,7 +843,7 @@ void ReturnStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void YieldExpression::accept0(Visitor *visitor)
+void YieldExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -847,7 +853,7 @@ void YieldExpression::accept0(Visitor *visitor)
 }
 
 
-void WithStatement::accept0(Visitor *visitor)
+void WithStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -857,7 +863,7 @@ void WithStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void SwitchStatement::accept0(Visitor *visitor)
+void SwitchStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -867,7 +873,7 @@ void SwitchStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void CaseBlock::accept0(Visitor *visitor)
+void CaseBlock::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(clauses, visitor);
@@ -878,7 +884,7 @@ void CaseBlock::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void CaseClauses::accept0(Visitor *visitor)
+void CaseClauses::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (CaseClauses *it = this; it; it = it->next) {
@@ -889,7 +895,7 @@ void CaseClauses::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void CaseClause::accept0(Visitor *visitor)
+void CaseClause::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -899,7 +905,7 @@ void CaseClause::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void DefaultClause::accept0(Visitor *visitor)
+void DefaultClause::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statements, visitor);
@@ -908,7 +914,7 @@ void DefaultClause::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void LabelledStatement::accept0(Visitor *visitor)
+void LabelledStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statement, visitor);
@@ -917,7 +923,7 @@ void LabelledStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ThrowStatement::accept0(Visitor *visitor)
+void ThrowStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -926,7 +932,7 @@ void ThrowStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TryStatement::accept0(Visitor *visitor)
+void TryStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statement, visitor);
@@ -937,7 +943,7 @@ void TryStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void Catch::accept0(Visitor *visitor)
+void Catch::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(patternElement, visitor);
@@ -947,7 +953,7 @@ void Catch::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void Finally::accept0(Visitor *visitor)
+void Finally::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statement, visitor);
@@ -956,20 +962,22 @@ void Finally::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void FunctionDeclaration::accept0(Visitor *visitor)
+void FunctionDeclaration::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(formals, visitor);
+        accept(typeAnnotation, visitor);
         accept(body, visitor);
     }
 
     visitor->endVisit(this);
 }
 
-void FunctionExpression::accept0(Visitor *visitor)
+void FunctionExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(formals, visitor);
+        accept(typeAnnotation, visitor);
         accept(body, visitor);
     }
 
@@ -981,9 +989,9 @@ FunctionExpression *FunctionExpression::asFunctionDefinition()
     return this;
 }
 
-QStringList FormalParameterList::formals() const
+BoundNames FormalParameterList::formals() const
 {
-    QStringList formals;
+    BoundNames formals;
     int i = 0;
     for (const FormalParameterList *it = this; it; it = it->next) {
         if (it->element) {
@@ -991,18 +999,18 @@ QStringList FormalParameterList::formals() const
             int duplicateIndex = formals.indexOf(name);
             if (duplicateIndex >= 0) {
                 // change the name of the earlier argument to enforce the lookup semantics from the spec
-                formals[duplicateIndex] += QLatin1String("#") + QString::number(i);
+                formals[duplicateIndex].id += QLatin1String("#") + QString::number(i);
             }
-            formals += name;
+            formals += {name, it->element->typeAnnotation};
         }
         ++i;
     }
     return formals;
 }
 
-QStringList FormalParameterList::boundNames() const
+BoundNames FormalParameterList::boundNames() const
 {
-    QStringList names;
+    BoundNames names;
     for (const FormalParameterList *it = this; it; it = it->next) {
         if (it->element)
             it->element->boundNames(&names);
@@ -1010,7 +1018,7 @@ QStringList FormalParameterList::boundNames() const
     return names;
 }
 
-void FormalParameterList::accept0(Visitor *visitor)
+void FormalParameterList::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (FormalParameterList *it = this; it && accepted; it = it->next) {
@@ -1035,7 +1043,7 @@ FormalParameterList *FormalParameterList::finish(QQmlJS::MemoryPool *pool)
     return front;
 }
 
-void Program::accept0(Visitor *visitor)
+void Program::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(statements, visitor);
@@ -1044,7 +1052,7 @@ void Program::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ImportSpecifier::accept0(Visitor *visitor)
+void ImportSpecifier::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
 
@@ -1052,7 +1060,7 @@ void ImportSpecifier::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ImportsList::accept0(Visitor *visitor)
+void ImportsList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (ImportsList *it = this; it; it = it->next) {
@@ -1063,7 +1071,7 @@ void ImportsList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NamedImports::accept0(Visitor *visitor)
+void NamedImports::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(importsList, visitor);
@@ -1072,7 +1080,7 @@ void NamedImports::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void FromClause::accept0(Visitor *visitor)
+void FromClause::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -1080,7 +1088,7 @@ void FromClause::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void NameSpaceImport::accept0(Visitor *visitor)
+void NameSpaceImport::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -1088,7 +1096,7 @@ void NameSpaceImport::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ImportClause::accept0(Visitor *visitor)
+void ImportClause::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(nameSpaceImport, visitor);
@@ -1098,7 +1106,7 @@ void ImportClause::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ImportDeclaration::accept0(Visitor *visitor)
+void ImportDeclaration::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(importClause, visitor);
@@ -1108,7 +1116,7 @@ void ImportDeclaration::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ExportSpecifier::accept0(Visitor *visitor)
+void ExportSpecifier::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
 
@@ -1117,7 +1125,7 @@ void ExportSpecifier::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ExportsList::accept0(Visitor *visitor)
+void ExportsList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (ExportsList *it = this; it; it = it->next) {
@@ -1128,7 +1136,7 @@ void ExportsList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ExportClause::accept0(Visitor *visitor)
+void ExportClause::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(exportsList, visitor);
@@ -1137,7 +1145,7 @@ void ExportClause::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ExportDeclaration::accept0(Visitor *visitor)
+void ExportDeclaration::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(fromClause, visitor);
@@ -1148,7 +1156,7 @@ void ExportDeclaration::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ESModule::accept0(Visitor *visitor)
+void ESModule::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(body, visitor);
@@ -1157,7 +1165,7 @@ void ESModule::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void DebuggerStatement::accept0(Visitor *visitor)
+void DebuggerStatement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -1165,7 +1173,7 @@ void DebuggerStatement::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiProgram::accept0(Visitor *visitor)
+void UiProgram::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(headers, visitor);
@@ -1175,19 +1183,23 @@ void UiProgram::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiPublicMember::accept0(Visitor *visitor)
+void UiPublicMember::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
+        // accept(memberType, visitor); // accept manually in visit if interested
         accept(statement, visitor);
         accept(binding, visitor);
+        // accept(parameters, visitor); // accept manually in visit if interested
     }
 
     visitor->endVisit(this);
 }
 
-void UiObjectDefinition::accept0(Visitor *visitor)
+void UiObjectDefinition::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(qualifiedTypeNameId, visitor);
         accept(initializer, visitor);
     }
@@ -1195,7 +1207,7 @@ void UiObjectDefinition::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiObjectInitializer::accept0(Visitor *visitor)
+void UiObjectInitializer::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(members, visitor);
@@ -1204,16 +1216,18 @@ void UiObjectInitializer::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiParameterList::accept0(Visitor *visitor)
+void UiParameterList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(type, visitor); // accept manually in visit if interested
     }
     visitor->endVisit(this);
 }
 
-void UiObjectBinding::accept0(Visitor *visitor)
+void UiObjectBinding::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(qualifiedId, visitor);
         accept(qualifiedTypeNameId, visitor);
         accept(initializer, visitor);
@@ -1222,9 +1236,10 @@ void UiObjectBinding::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiScriptBinding::accept0(Visitor *visitor)
+void UiScriptBinding::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(qualifiedId, visitor);
         accept(statement, visitor);
     }
@@ -1232,9 +1247,10 @@ void UiScriptBinding::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiArrayBinding::accept0(Visitor *visitor)
+void UiArrayBinding::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(qualifiedId, visitor);
         accept(members, visitor);
     }
@@ -1242,7 +1258,7 @@ void UiArrayBinding::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiObjectMemberList::accept0(Visitor *visitor)
+void UiObjectMemberList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (UiObjectMemberList *it = this; it; it = it->next)
@@ -1252,7 +1268,7 @@ void UiObjectMemberList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiArrayMemberList::accept0(Visitor *visitor)
+void UiArrayMemberList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         for (UiArrayMemberList *it = this; it; it = it->next)
@@ -1262,24 +1278,55 @@ void UiArrayMemberList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiQualifiedId::accept0(Visitor *visitor)
+void UiQualifiedId::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(next, visitor) // accept manually in visit if interested
     }
 
     visitor->endVisit(this);
 }
 
-void UiImport::accept0(Visitor *visitor)
+void Type::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        accept(typeId, visitor);
+        accept(typeArguments, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
+void TypeArgumentList::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        for (TypeArgumentList *it = this; it; it = it->next)
+            accept(it->typeId, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
+void TypeAnnotation::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        accept(type, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
+void UiImport::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(importUri, visitor);
+        // accept(version, visitor); // accept manually in visit if interested
     }
 
     visitor->endVisit(this);
 }
 
-void UiPragma::accept0(Visitor *visitor)
+void UiPragma::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -1287,7 +1334,7 @@ void UiPragma::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void UiHeaderItemList::accept0(Visitor *visitor)
+void UiHeaderItemList::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (UiHeaderItemList *it = this; it && accepted; it = it->next) {
@@ -1300,25 +1347,27 @@ void UiHeaderItemList::accept0(Visitor *visitor)
 }
 
 
-void UiSourceElement::accept0(Visitor *visitor)
+void UiSourceElement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(sourceElement, visitor);
     }
 
     visitor->endVisit(this);
 }
 
-void UiEnumDeclaration::accept0(Visitor *visitor)
+void UiEnumDeclaration::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
         accept(members, visitor);
     }
 
     visitor->endVisit(this);
 }
 
-void UiEnumMemberList::accept0(Visitor *visitor)
+void UiEnumMemberList::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
     }
@@ -1326,7 +1375,7 @@ void UiEnumMemberList::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void TaggedTemplate::accept0(Visitor *visitor)
+void TaggedTemplate::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(base, visitor);
@@ -1336,17 +1385,18 @@ void TaggedTemplate::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void PatternElement::accept0(Visitor *visitor)
+void PatternElement::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(bindingTarget, visitor);
+        accept(typeAnnotation, visitor);
         accept(initializer, visitor);
     }
 
     visitor->endVisit(this);
 }
 
-void PatternElement::boundNames(QStringList *names)
+void PatternElement::boundNames(BoundNames *names)
 {
     if (bindingTarget) {
         if (PatternElementList *e = elementList())
@@ -1354,11 +1404,11 @@ void PatternElement::boundNames(QStringList *names)
         else if (PatternPropertyList *p = propertyList())
             p->boundNames(names);
     } else {
-        names->append(bindingIdentifier.toString());
+        names->append({bindingIdentifier.toString(), typeAnnotation});
     }
 }
 
-void PatternElementList::accept0(Visitor *visitor)
+void PatternElementList::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (PatternElementList *it = this; it && accepted; it = it->next) {
@@ -1371,7 +1421,7 @@ void PatternElementList::accept0(Visitor *visitor)
     }
 }
 
-void PatternElementList::boundNames(QStringList *names)
+void PatternElementList::boundNames(BoundNames *names)
 {
     for (PatternElementList *it = this; it; it = it->next) {
         if (it->element)
@@ -1379,23 +1429,24 @@ void PatternElementList::boundNames(QStringList *names)
     }
 }
 
-void PatternProperty::accept0(Visitor *visitor)
+void PatternProperty::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(name, visitor);
         accept(bindingTarget, visitor);
+        accept(typeAnnotation, visitor);
         accept(initializer, visitor);
     }
 
     visitor->endVisit(this);
 }
 
-void PatternProperty::boundNames(QStringList *names)
+void PatternProperty::boundNames(BoundNames *names)
 {
     PatternElement::boundNames(names);
 }
 
-void PatternPropertyList::accept0(Visitor *visitor)
+void PatternPropertyList::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (PatternPropertyList *it = this; it && accepted; it = it->next) {
@@ -1406,13 +1457,13 @@ void PatternPropertyList::accept0(Visitor *visitor)
     }
 }
 
-void PatternPropertyList::boundNames(QStringList *names)
+void PatternPropertyList::boundNames(BoundNames *names)
 {
     for (PatternPropertyList *it = this; it; it = it->next)
         it->property->boundNames(names);
 }
 
-void ComputedPropertyName::accept0(Visitor *visitor)
+void ComputedPropertyName::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(expression, visitor);
@@ -1421,7 +1472,7 @@ void ComputedPropertyName::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ClassExpression::accept0(Visitor *visitor)
+void ClassExpression::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(heritage, visitor);
@@ -1436,7 +1487,7 @@ ClassExpression *ClassExpression::asClassDefinition()
     return this;
 }
 
-void ClassDeclaration::accept0(Visitor *visitor)
+void ClassDeclaration::accept0(BaseVisitor *visitor)
 {
     if (visitor->visit(this)) {
         accept(heritage, visitor);
@@ -1446,7 +1497,7 @@ void ClassDeclaration::accept0(Visitor *visitor)
     visitor->endVisit(this);
 }
 
-void ClassElementList::accept0(Visitor *visitor)
+void ClassElementList::accept0(BaseVisitor *visitor)
 {
     bool accepted = true;
     for (ClassElementList *it = this; it && accepted; it = it->next) {
@@ -1475,8 +1526,77 @@ LeftHandSideExpression *LeftHandSideExpression::leftHandSideExpressionCast()
     return this;
 }
 
+void UiVersionSpecifier::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+    }
+    visitor->endVisit(this);
+}
+
+QString Type::toString() const
+{
+    QString result;
+    toString(&result);
+    return result;
+}
+
+void Type::toString(QString *out) const
+{
+    for (QQmlJS::AST::UiQualifiedId *it = typeId; it; it = it->next) {
+        out->append(it->name);
+
+        if (it->next)
+            out->append(QLatin1Char('.'));
+    }
+
+    if (typeArguments) {
+        out->append(QLatin1Char('<'));
+        if (auto subType = static_cast<TypeArgumentList*>(typeArguments)->typeId)
+            subType->toString(out);
+        out->append(QLatin1Char('>'));
+    };
+}
+
+void UiInlineComponent::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        // accept(annotations, visitor); // accept manually in visit if interested
+        accept(component, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
+void UiRequired::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+    }
+
+    visitor->endVisit(this);
+}
+
+void UiAnnotationList::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        for (UiAnnotationList *it = this; it; it = it->next)
+            accept(it->annotation, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
+void UiAnnotation::accept0(BaseVisitor *visitor)
+{
+    if (visitor->visit(this)) {
+        accept(qualifiedTypeNameId, visitor);
+        accept(initializer, visitor);
+    }
+
+    visitor->endVisit(this);
+}
+
 } } // namespace QQmlJS::AST
 
-QT_QML_END_NAMESPACE
+QT_END_NAMESPACE
 
 
