@@ -3,16 +3,16 @@
 
 /*---
 esid: sec-array.prototype.map
-es5id: 15.4.4.19-5-11
-description: Array.prototype.map - String object can be used as thisArg
+es5id: 15.4.4.19-4-12
+description: Array.prototype.map - 'callbackfn' is a function
 ---*/
 
-var objString = new String();
-
 function callbackfn(val, idx, obj) {
-  return this === objString;
+  return val > 10;
 }
 
-var testResult = [11].map(callbackfn, objString);
+var testResult = [11, 9].map(callbackfn);
 
+assert.sameValue(testResult.length, 2, 'testResult.length');
 assert.sameValue(testResult[0], true, 'testResult[0]');
+assert.sameValue(testResult[1], false, 'testResult[1]');
