@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.12.2-2-10
+es5id: 15.12.2-2-4
 description: >
-    JSON.parse - parsing an object where property value middles with a
-    null character
+    JSON.parse - parsing an object where property name starts and ends
+    with a null character
 ---*/
 
 var nullChars = new Array();
@@ -44,6 +44,6 @@ nullChars[31] = '\"\u001F\"';
 
 for (var index in nullChars) {
   assert.throws(SyntaxError, function() {
-    var obj = JSON.parse('{ "name" : ' + "Jo" + nullChars[index] + "hn" + ' } ');
+    var obj = JSON.parse('{' + nullChars[index] + "name" + nullChars[index] + ' : "John" } ');
   });
 }
