@@ -2,12 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-24
+es5id: 15.2.3.2-2-26
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (Number object)
+    (RegExp object)
 ---*/
 
-var obj = new Number(-3);
+var obj = new RegExp();
 
-assert.sameValue(Object.getPrototypeOf(obj), Number.prototype, 'Object.getPrototypeOf(obj)');
+assert.sameValue(Object.getPrototypeOf(obj), RegExp.prototype, 'Object.getPrototypeOf(obj)');
