@@ -2,10 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.5.4.20-4-37
+es5id: 15.5.4.20-4-21
 description: >
     String.prototype.trim handles whitepace and lineterminators
-    (ab\u000Cc)
+    (\u000Cabc\u000C)
 ---*/
 
-assert.sameValue("ab\u000Cc".trim(), "ab\u000Cc", '"ab\u000Cc".trim()');
+assert.sameValue("\u000Cabc\u000C".trim(), "abc", '"\u000Cabc\u000C".trim()');
