@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 11.13.1-4-6-s
+es5id: 11.13.1-4-14-s
 description: >
     simple assignment throws TypeError if LeftHandSide is a readonly
-    property in strict mode (Function.length)
+    property in strict mode (Number.MAX_VALUE)
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(TypeError, function() {
-    Function.length = 42;
+    Number.MAX_VALUE = 42;
 });
