@@ -5,14 +5,19 @@
 esid: prod-RegularExpressionBackslashSequence
 info: |
   RegularExpressionBackslashSequence ::
-  \ RegularExpressionNonTerminator
+    \ RegularExpressionNonTerminator
 
   RegularExpressionNonTerminator ::
     SourceCharacter but not LineTerminator
 
-description: >
-  A RegularExpressionBackslashSequence may not contain a <LF>
+  LineTerminator ::
+    <LF>
+    <CR>
+    <LS>
+    <PS>
 
+description: >
+  A regular expression may not contain a <LF> as a SourceCharacter
 negative:
   phase: parse
   type: SyntaxError
@@ -20,5 +25,5 @@ negative:
 
 throw "Test262: This statement should not be evaluated.";
 
-/\
+/a\
 /
