@@ -2,14 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-3-s
+es5id: 12.2.1-27-s
 description: >
-    eval - a function expr declaring a var named 'eval' throws
-    SyntaxError in strict mode
+    eval as local var identifier assigned to throws SyntaxError in
+    strict mode
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-    eval('(function () { var eval; })');
+    eval('function foo() { var eval = 42, a;}');
 });
