@@ -36,28 +36,36 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '12.6.3-1.js';
+gTestfile = '12.6.2-8.js';
 
 /**
-   File Name:          12.6.3-1.js
-   ECMA Section:       12.6.3 The for...in Statement
-   Description:
+   File Name:          12.6.2-8.js
+   ECMA Section:       12.6.2 The for Statement
+
+   1. first expression is present.
+   2. second expression is present
+   3. third expression is present
+
 
    Author:             christine@netscape.com
-   Date:               11 september 1997
+   Date:               15 september 1997
 */
-
-var SECTION = "12.6.3-1";
+var SECTION = "12.6.2-8";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "The for..in statement";
+var TITLE   = "The for statement";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION,
-	      "var x; Number.prototype.foo = 34; for ( j in 7 ) x = j; x",
-	      "foo",
-	      eval("var x; Number.prototype.foo = 34; for ( j in 7 ){x = j;} x") );
+new TestCase( SECTION, "for statement",  256,     testprogram() );
 
 test();
 
+function testprogram() {
+  var myVar;
+
+  for ( myVar=2;  myVar < 256; myVar *= myVar ) {
+  }
+
+  return myVar;
+}
